#-*- coding: utf-8 -*-

#**********************************************
# R2spec 
#
# Class file for the class Description
#
# Distributed under License GPLv3 or later
# You can find a copy of this license on the website
# http://www.gnu.org/licenses/gpl.html
#
#**********************************************

import os, re, shutil, string

class Description:
    '''
    Description class.
        Creates and handle the object Description
    '''

    def __init__(self):
        ''' Creates the object '''
        self.license = None
        self.version = None
        self.description = None
        self.summary = None
        self.file = None

    def setVersion(self):
        ''' Set the version of the package '''
        # Retrieve the Version number
        try:
            self.version = self.file['version'].replace('-','.')
        except KeyError, err:
            print "No version set"
            self.version = ""


    def setLicense(self):
        ''' Returns the license of the package '''
        try:
            self.license = self.file['license']
        except IndexError, err:
            print "No license set"
            self.license = ""


    def setDescription(self):
        ''' 
        Returns the description of the package. 
        Catch the description in one or several lines 
        '''
        try:
            self.description = self.formatDescription(self.file['description'])
            print self.description
        except KeyError, err:
            print 'No Description available'
            self.description = ""


    def setSummary(self):
        ''' 
        Returns the summary of the package. 
        Catch the title in one or several lines 
        '''
        try:
            self.summary = self.file['title'][:56].capitalize()
            if self.summary.endswith('.'):
                self.summary = self.summary[:-1]
        except KeyError, err:
            print 'No Title available'
            self.summary = ""


    def setRequires(self):
        ''' 
        Returns the required libraries of the package.
        Catch the requirement in one or more lines.
        '''
        # Characters to change and to change to 
        char = {'  ':' ', 
                '\r':'', 
                '(' :'', 
                ')' :'', 
                ',' :'',
                }
        # Package that should not be taken into account for the dependencies
        ignorelist = ['R']
        # Regular expression used to determine whether the string is a version number
        versionmotif = re.compile('\d\.\d\.?\d?')
        
        # Retrieve the Depends
        try:
            depends = self.file['depends']
            for c in char.keys():
                depends = depends.replace(c, char[c])
            
            # Check that the dependencies are not ''
            deplist = depends.split(' ')
            cnt = 0
            for dep in deplist:
                deplist[cnt] = dep.strip()
                if dep.strip() == "":
                    del deplist[cnt]
                cnt = cnt + 1
            
            # Add the prefix R- to the dependencies
            # Check if the dependencies does not contains > or = or a number
            cnt = 0
            while cnt < len(deplist):
                if  not ">" in deplist[cnt] \
                    and not "=" in deplist[cnt] \
                    and len( versionmotif.findall(deplist[cnt]) ) == 0 \
                    and deplist[cnt] not in ignorelist:
                        deplist[cnt] = "R-%s" %deplist[cnt]
                cnt = cnt + 1
            depends = " ".join(deplist)
            
            self.depends = depends.strip()
        except KeyError, err:
            print 'No Depends set'
            self.depends = ""

        # Retrieve the Suggests
        try:
            suggests = self.file['suggests']
            for c in char.keys():
                suggests = suggests.replace(c, char[c])
            
            # Check that the dependencies are not ''
            suglist = suggests.split(' ')
            cnt = 0
            for sug in suglist:
                suglist[cnt] = sug.strip()
                if sug.strip() == "":
                    del suglist[cnt]
                cnt = cnt + 1

            # Add the prefix R- to the dependencies
            # Check if the dependencies does not contains > or = or a number
            cnt = 0
            while cnt < len(suglist):
                if  not ">" in suglist[cnt] \
                    and not "=" in suglist[cnt] \
                    and len( versionmotif.findall(suglist[cnt]) ) == 0 \
                    and suglist[cnt] not in ignorelist:
                        suglist[cnt] = "R-%s" %suglist[cnt]
                cnt = cnt + 1
            
            suggests = " ".join(suglist)
            
            self.suggests = suggests.strip()
        except KeyError, err:
            print 'No Suggests set'
            self.suggests = ""

  
    def setDescriptionFile(self, package):
        ''' Returns the description file of the package '''

        # Save the top directory
        original_directory = os.getcwd()

        # Change directory and load the DESCRIPTION file in memory
        os.chdir("./tmp/"+package.name)
        self.file = self.parseConfig('DESCRIPTION')

        # Comes back to the original directory
        os.chdir(original_directory)


    def parseConfig(self, filename):
        ''' Returns a dictionnary of all the options available '''
        # Character used in the DESCRIPTION file
        COMMENT_CHAR = '#' # char for comments
        OPTION_CHAR =  ':' # char used to separate option from value
        
        options = {}
        f = open(filename)
        for line in f:
            # First, remove comments:
            if COMMENT_CHAR in line:
                # split on comment char, keep only the part before
                line, comment = unicode(line, errors='ignore').split(COMMENT_CHAR, 1)
            # Second, find lines with an option=value:
            if OPTION_CHAR in line:
                # split on option char:
                option, value = unicode(line, errors='ignore').split(OPTION_CHAR, 1)
                # strip spaces:
                option = option.strip().lower()
                value = value.strip()
                # store in dictionary:
                options[option] = value.encode('UTF-8')
            else:
                values = unicode(line, errors='ignore').split(" ")
                for value in values:
                    if value.strip() != "":
                        options[option] += " " +value.strip().encode('UTF-8')
                
        f.close()
        return options
    
    def formatDescription(self, description):
        ''' Format the description as required by rpm '''
        step = 75
        cnt = 0
        out = []
        char = 0
        while cnt < len(description) and char < description.rfind(" "):
            if len(description[cnt:]) >= step:
                char = description[cnt : cnt + step ].rfind(" ")  + cnt
                out.append(description[cnt:char])
                cnt = char + 1
            else:
                out.append(description[cnt:])
                cnt += len(description[cnt:])

        return "\n".join(out)
